RunScope

Struct RunScope 

pub struct RunScope { /* private fields */ }
Guard returned by Client::run_scope. Restores the client’s previous ambient run id on Drop, or after RunScope::end_with_reflect.

Implementations

Source

impl RunScope

pub fn run_id(&self) -> &str

The run id this scope pinned as the client’s ambient run id.

pub async fn end_with_reflect(self) -> Result<Value>

Explicitly end the scope with a reflection pass over the scoped run, then restore the previous ambient run id. Use this instead of relying on Drop when you want lessons distilled at the end of the run — Drop cannot await.